#include <SoftwareSerial.h> 
#include <Servo.h>
#include <HX711_ADC.h>
#if defined(ESP8266)|| defined(ESP32) || defined(AVR)
#include <EEPROM.h>
#endif

Servo myservo;
SoftwareSerial MyBlue(2, 3); // RX | TX 
int flag = 2; 

#define buttonBlue 6
#define buttonBlack 7
#define buttonRed 8
#define pump 11
#define waterSensorPower 10
#define waterSensorPin A0
#define HX711_dout 4
#define HX711_sck  5

HX711_ADC LoadCell(HX711_dout, HX711_sck);

const int calVal_eepromAdress = 0;
unsigned long t = 0;

float weight = 150;
int stable = 0;
int releaseFood = 0;


int buttonBlueState = 0;
int buttonBlackState = 0;
int buttonRedState = 0;
int mode = 0;

void setup() 
{   
  Serial.begin(9600); 
  MyBlue.begin(9600); 

  pinMode(buttonBlue, INPUT);
  pinMode(buttonRed, INPUT);
  pinMode(buttonBlack, INPUT);

  pinMode(pump, OUTPUT);
  myservo.attach(9);

  Serial.println();
  Serial.println("Starting...");

  LoadCell.begin();
  
  float calibrationValue;
  calibrationValue = 696.0;
  
  EEPROM.get(calVal_eepromAdress, calibrationValue);

  unsigned long stabilizingtime = 2000;
  boolean _tare = true;
  LoadCell.start(stabilizingtime, _tare);
  if (LoadCell.getTareTimeoutFlag()) {
    Serial.println("Timeout, check MCU>HX711 wiring and pin designations");
    while (1);
  }
  else {
    LoadCell.setCalFactor(calibrationValue);
    Serial.println("Startup is complete");
  }
} 

int isStable(float i, float weight) {

  float dif;
  
  if(i > weight) {
    dif = i - weight;
  }
  else {
    dif = weight - i;
  }

  if(dif < 0.02)
    return 1;
  else {
    return 0;
  }
  
}

void weightSensorFunc(){

  static boolean newDataReady = 0;
  const int serialPrintInterval = 0;

  if (LoadCell.update()) 
    newDataReady = true;

  if (newDataReady) {
    float i = LoadCell.getData();

    if(isStable(i, weight)) {
      if(weight > 45.00 && weight < 65.00) {
        startMotor();
      }
      else if(weight > 120.00) {
        stopMotor();
      }
        
    }
      
    weight = i;
      
    Serial.print("Load_cell output val: ");
    Serial.println(i);
    newDataReady = 0;
    t = millis();
  }

  if (Serial.available() > 0) {
    char inByte = Serial.read();
    if (inByte == 't') LoadCell.tareNoDelay();
  }

  if (LoadCell.getTareStatus() == true) {
    Serial.println("Tare complete");
  }
}

void startMotor(){
  myservo.write(180);
}

void stopMotor() {
  myservo.write(90);
}

void startPump() {
  digitalWrite(pump, HIGH);
}

void stopPump() {
  digitalWrite(pump, LOW);
}

void waterSensorFunc() {

  int level = readSensor();

  if(level < 300) {
    startPump();
    delay(2000);
    stopPump();
    delay(2000);
  }
  
  Serial.print("Water level: ");
  Serial.println(level);
  
  delay(1000);
}

int val = 0;

int readSensor() {
  
  digitalWrite(waterSensorPower, HIGH);
  delay(10);
  val = analogRead(waterSensorPin);
  digitalWrite(waterSensorPower, LOW);
  return val;
}

void loop() 
{ 

  buttonBlueState = digitalRead(buttonBlue);
  buttonRedState = digitalRead(buttonRed);
  buttonBlackState = digitalRead(buttonBlack);

  if(buttonBlackState) {
    mode = (mode + 1) % 3;
  }
     
  if(mode == 0) {

    if(buttonRedState) {
      startMotor();
    } else {
      stopMotor();
    }

    if(buttonBlueState) {
      startPump();
    } else {
      stopPump();
    }
  }

  if(mode == 1) {

    if (MyBlue.available()) 
      flag = MyBlue.read(); 
      
    if (flag == 48)
    { 
      startPump();
    } 
    if (flag == 49)
    { 
      stopPump();
    } 
  }

  if(mode == 2) {
    weightSensorFunc();
    waterSensorFunc();
  }

}
